@@ -11,11 +11,14 @@
 # under the License.
 
 import abc
+import typing as ty
 
 try:
-    from lxml import etree
+    # explicitly re-export symbol
+    # https://mypy.readthedocs.io/en/stable/command_line.html#cmdoption-mypy-no-implicit-reexport
+    from lxml import etree as etree
 except ImportError:
-    etree = None
+    etree = None  # type: ignore[assignment]
 
 import requests
 import requests.auth
@@ -23,12 +26,13 @@ import requests.auth
 from keystoneauth1 import access
 from keystoneauth1 import exceptions
 from keystoneauth1.identity import v3
+from keystoneauth1 import session as ks_session
 
 _PAOS_NAMESPACE = 'urn:liberty:paos:2003-08'
 _ECP_NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:profiles:SSO:ecp'
 _PAOS_HEADER = 'application/vnd.paos+xml'
 
-_PAOS_VER = 'ver="%s";"%s"' % (_PAOS_NAMESPACE, _ECP_NAMESPACE)
+_PAOS_VER = f'ver="{_PAOS_NAMESPACE}";"{_ECP_NAMESPACE}"'
 
 _XML_NAMESPACES = {
     'ecp': _ECP_NAMESPACE,
@@ -37,7 +41,6 @@ _XML_NAMESPACES = {
 }
 
 _XBASE = '/S:Envelope/S:Header/'
-
 _XPATH_SP_RELAY_STATE = '//ecp:RelayState'
 _XPATH_SP_CONSUMER_URL = _XBASE + 'paos:Request/@responseConsumerURL'
 _XPATH_IDP_CONSUMER_URL = _XBASE + 'ecp:Response/@AssertionConsumerServiceURL'
@@ -68,19 +71,25 @@ class ConsumerMismatch(SamlException):
     """The SP and IDP consumers do not match."""
 
 
-def _response_xml(response, name):
+def _response_xml(response: requests.Response, name: str) -> etree._Element:
     try:
         return etree.XML(response.content)
     except etree.XMLSyntaxError as e:
-        msg = 'SAML2: Error parsing XML returned from %s: %s' % (name, e)
+        msg = f'SAML2: Error parsing XML returned from {name}: {e}'
         raise InvalidResponse(msg)
 
 
-def _str_from_xml(xml, path):
-    l = xml.xpath(path, namespaces=_XML_NAMESPACES)
-    if len(l) != 1:
-        raise IndexError('%s should provide a single element list' % path)
-    return l[0]
+def _str_from_xml(xml: etree._Element, path: str) -> str:
+    li = xml.xpath(path, namespaces=_XML_NAMESPACES)
+    if len(li) != 1:
+        raise IndexError(f'{path} should provide a single element list')
+    result = li[0]
+    return str(result)  # case from _ElementUnicodeResult
+
+
+_PreparedRequestT = ty.TypeVar(
+    '_PreparedRequestT', bound=requests.PreparedRequest
+)
 
 
 class _SamlAuth(requests.auth.AuthBase):
@@ -114,91 +123,130 @@ class _SamlAuth(requests.auth.AuthBase):
         federated user is a member of.
     """
 
-    def __init__(self, identity_provider_url, requests_auth):
-        super(_SamlAuth, self).__init__()
+    def __init__(
+        self,
+        identity_provider_url: str,
+        requests_auth: ty.Union[None, requests.auth.AuthBase, tuple[str, str]],
+    ):
+        super().__init__()
         self.identity_provider_url = identity_provider_url
         self.requests_auth = requests_auth
 
-    def __call__(self, request):
+    def __call__(self, request: _PreparedRequestT) -> _PreparedRequestT:
         try:
             accept = request.headers['Accept']
         except KeyError:
             request.headers['Accept'] = _PAOS_HEADER
         else:
-            request.headers['Accept'] = ';'.join([accept, _PAOS_HEADER])
+            request.headers['Accept'] = ','.join([accept, _PAOS_HEADER])
 
         request.headers['PAOS'] = _PAOS_VER
-        request.register_hook('response', self._handle_response)
+        request.register_hook('response', self._handle_response)  # type: ignore[no-untyped-call]
         return request
 
-    def _handle_response(self, response, **kwargs):
-        if (response.status_code == 200 and
-                response.headers.get('Content-Type') == _PAOS_HEADER):
+    def _handle_response(
+        self, response: requests.Response, **kwargs: ty.Any
+    ) -> requests.Response:
+        if (
+            response.status_code == 200
+            and response.headers.get('Content-Type') == _PAOS_HEADER
+        ):
             response = self._ecp_retry(response, **kwargs)
 
         return response
 
-    def _ecp_retry(self, sp_response, **kwargs):
+    def _ecp_retry(
+        self, sp_response: requests.Response, **kwargs: ty.Any
+    ) -> requests.Response:
         history = [sp_response]
 
-        def send(*send_args, **send_kwargs):
-            req = requests.Request(*send_args, **send_kwargs)
+        def send(
+            method: str,
+            url: str,
+            headers: ty.Mapping[str, str],
+            data: bytes,
+            auth: ty.Union[
+                None, requests.auth.AuthBase, tuple[str, str]
+            ] = None,
+            cookies: ty.Optional[requests.cookies.RequestsCookieJar] = None,
+        ) -> requests.Response:
+            req = requests.Request(
+                method=method,
+                url=url,
+                headers=headers,
+                data=data,
+                auth=auth,
+                cookies=cookies,
+            )
             return sp_response.connection.send(req.prepare(), **kwargs)
 
         authn_request = _response_xml(sp_response, 'Service Provider')
-        relay_state = _str_from_xml(authn_request, _XPATH_SP_RELAY_STATE)
+        relay_state = authn_request.xpath(
+            _XPATH_SP_RELAY_STATE, namespaces=_XML_NAMESPACES
+        )[0]
         sp_consumer_url = _str_from_xml(authn_request, _XPATH_SP_CONSUMER_URL)
 
         authn_request.remove(authn_request[0])
 
-        idp_response = send('POST',
-                            self.identity_provider_url,
-                            headers={'Content-type': 'text/xml'},
-                            data=etree.tostring(authn_request),
-                            auth=self.requests_auth)
+        idp_response = send(
+            'POST',
+            self.identity_provider_url,
+            headers={'Content-type': 'text/xml'},
+            data=etree.tostring(authn_request),
+            auth=self.requests_auth,
+        )
         history.append(idp_response)
 
         authn_response = _response_xml(idp_response, 'Identity Provider')
-        idp_consumer_url = _str_from_xml(authn_response,
-                                         _XPATH_IDP_CONSUMER_URL)
+        idp_consumer_url = _str_from_xml(
+            authn_response, _XPATH_IDP_CONSUMER_URL
+        )
 
         if sp_consumer_url != idp_consumer_url:
             # send fault message to the SP, discard the response
-            send('POST',
-                 sp_consumer_url,
-                 data=_SOAP_FAULT,
-                 headers={'Content-Type': _PAOS_HEADER})
+            send(
+                'POST',
+                sp_consumer_url,
+                data=_SOAP_FAULT.encode('utf-8'),
+                headers={'Content-Type': _PAOS_HEADER},
+            )
 
             # prepare error message and raise an exception.
-            msg = ('Consumer URLs from Service Provider %(service_provider)s '
-                   '%(sp_consumer_url)s and Identity Provider '
-                   '%(identity_provider)s %(idp_consumer_url)s are not equal')
+            msg = (
+                'Consumer URLs from Service Provider %(service_provider)s '
+                '%(sp_consumer_url)s and Identity Provider '
+                '%(identity_provider)s %(idp_consumer_url)s are not equal'
+            )
             msg = msg % {
                 'service_provider': sp_response.request.url,
                 'sp_consumer_url': sp_consumer_url,
                 'identity_provider': self.identity_provider_url,
-                'idp_consumer_url': idp_consumer_url
+                'idp_consumer_url': idp_consumer_url,
             }
 
             raise ConsumerMismatch(msg)
 
+        # FIXME(stephenfin): We need a better type here
         authn_response[0][0] = relay_state
 
         # idp_consumer_url is the URL on the SP that handles the ECP body
         # returned and creates an authenticated session.
-        final_resp = send('POST',
-                          idp_consumer_url,
-                          headers={'Content-Type': _PAOS_HEADER},
-                          cookies=idp_response.cookies,
-                          data=etree.tostring(authn_response))
+        final_resp = send(
+            'POST',
+            idp_consumer_url,
+            headers={'Content-Type': _PAOS_HEADER},
+            cookies=idp_response.cookies,
+            data=etree.tostring(authn_response),
+        )
 
         history.append(final_resp)
 
         # the SP should then redirect us back to the original URL to retry the
         # original request.
-        if final_resp.status_code in (requests.codes.found,
-                                      requests.codes.other):
-
+        if final_resp.status_code in (
+            requests.codes.found,
+            requests.codes.other,
+        ):
             # Consume content and release the original connection
             # to allow our new request to reuse the same one.
             sp_response.content
@@ -216,31 +264,63 @@ class _SamlAuth(requests.auth.AuthBase):
 
 
 class _FederatedSaml(v3.FederationBaseAuth):
-
-    def __init__(self, auth_url, identity_provider, protocol,
-                 identity_provider_url, **kwargs):
-        super(_FederatedSaml, self).__init__(auth_url,
-                                             identity_provider,
-                                             protocol,
-                                             **kwargs)
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        identity_provider_url: str,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
+        super().__init__(
+            auth_url,
+            identity_provider,
+            protocol,
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
         self.identity_provider_url = identity_provider_url
 
     @abc.abstractmethod
-    def get_requests_auth(self):
-        raise NotImplemented()
+    def get_requests_auth(self) -> requests.auth.AuthBase:
+        raise NotImplementedError()
 
-    def get_unscoped_auth_ref(self, session, **kwargs):
+    def get_unscoped_auth_ref(
+        self, session: ks_session.Session
+    ) -> access.AccessInfoV3:
         method = self.get_requests_auth()
         auth = _SamlAuth(self.identity_provider_url, method)
 
         try:
-            resp = session.get(self.federated_token_url,
-                               requests_auth=auth,
-                               authenticated=False)
+            resp = session.get(
+                self.federated_token_url,
+                requests_auth=auth,
+                authenticated=False,
+            )
         except SamlException as e:
             raise exceptions.AuthorizationFailure(str(e))
 
-        return access.create(resp=resp)
+        access_info = access.create(resp=resp)
+        assert isinstance(access_info, access.AccessInfoV3)  # nosec B101
+        return access_info
 
 
 class Password(_FederatedSaml):
@@ -287,15 +367,44 @@ class Password(_FederatedSaml):
 
     """
 
-    def __init__(self, auth_url, identity_provider, protocol,
-                 identity_provider_url, username, password, **kwargs):
-        super(Password, self).__init__(auth_url,
-                                       identity_provider,
-                                       protocol,
-                                       identity_provider_url,
-                                       **kwargs)
+    def __init__(
+        self,
+        auth_url: str,
+        identity_provider: str,
+        protocol: str,
+        identity_provider_url: str,
+        username: str,
+        password: str,
+        *,
+        trust_id: ty.Optional[str] = None,
+        system_scope: ty.Optional[str] = None,
+        domain_id: ty.Optional[str] = None,
+        domain_name: ty.Optional[str] = None,
+        project_id: ty.Optional[str] = None,
+        project_name: ty.Optional[str] = None,
+        project_domain_id: ty.Optional[str] = None,
+        project_domain_name: ty.Optional[str] = None,
+        reauthenticate: bool = True,
+        include_catalog: bool = True,
+    ):
+        super().__init__(
+            auth_url,
+            identity_provider,
+            protocol,
+            identity_provider_url,
+            trust_id=trust_id,
+            system_scope=system_scope,
+            domain_id=domain_id,
+            domain_name=domain_name,
+            project_id=project_id,
+            project_name=project_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name,
+            reauthenticate=reauthenticate,
+            include_catalog=include_catalog,
+        )
         self.username = username
         self.password = password
 
-    def get_requests_auth(self):
+    def get_requests_auth(self) -> requests.auth.AuthBase:
         return requests.auth.HTTPBasicAuth(self.username, self.password)
